/*
 *  upkg: tool for manipulating Unreal Tournament packages.
 *  Copyright (C) 2009 Nick Bowler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <limits.h>

#include "pack.h"

/* Unsigned integer packing. */
#define DEFPACK_BE(bits, type) void pack_ ## bits ## _be ( \
	unsigned char *out, type v \
) { \
	unsigned i; \
	for (i = 1; i <= bits/8; i++) { \
		out[bits/8 - i] = v % 256; \
		v /= 256; \
	} \
}

#define DEFPACK_LE(bits, type) void pack_ ## bits ## _le ( \
	unsigned char *out, type v \
) { \
	unsigned i; \
	for (i = 0; i < bits/8; i++) { \
		out[i] = v % 256; \
		v /= 256; \
	} \
}

DEFPACK_BE(16, unsigned short)
DEFPACK_BE(32, unsigned long)
#ifdef ULLONG_MAX
DEFPACK_BE(64, unsigned long long)
#endif

DEFPACK_LE(16, unsigned short)
DEFPACK_LE(32, unsigned long)
#ifdef ULLONG_MAX
DEFPACK_LE(64, unsigned long long)
#endif

#define DEFUNPACK_BE(bits, type) type unpack_ ## bits ## _be ( \
	const unsigned char *in \
) { \
	type v = 0; \
	unsigned i; \
	for (i = 0; i < bits/8; i++) { \
		v *= 256; \
		v += in[i]; \
	} \
	return v; \
}

#define DEFUNPACK_LE(bits, type) type unpack_ ## bits ## _le ( \
	const unsigned char *in \
) { \
	type v = 0; \
	unsigned i; \
	for (i = 1; i <= bits/8; i++) { \
		v *= 256; \
		v += in[bits/8 - i]; \
	} \
	return v; \
}

DEFUNPACK_BE(16, unsigned short)
DEFUNPACK_BE(32, unsigned long)
#ifdef ULLONG_MAX
DEFUNPACK_BE(64, unsigned long long)
#endif

DEFUNPACK_LE(16, unsigned short)
DEFUNPACK_LE(32, unsigned long)
#ifdef ULLONG_MAX
DEFUNPACK_LE(64, unsigned long long)
#endif

/*
 * Two's complement signed integer packing.  This is unlikely to work on
 * systems that don't themselves use two's complement.
 */

#define DEFUNPACK_SBE(bits, max, type) type unpack_s ## bits ## _be ( \
	const unsigned char *in \
) { \
	type v = 0; \
	unsigned i; \
	int sign = (in[0] & 0x80) ? 1 : 0; \
	for (i = 0; i < bits/8; i++) { \
		v *= 256; \
		v += in[i] & (i == 0 ? 0x7f : 0xff); \
	} \
	return sign*(-max-1) + v; \
}

#define DEFUNPACK_SLE(bits, max, type) type unpack_s ## bits ## _le ( \
	const unsigned char *in \
) { \
	type v = 0; \
	unsigned i; \
	int sign = (in[bits/8 - 1] & 0x80) ? 1 : 0; \
	for (i = 1; i <= bits/8; i++) { \
		v *= 256; \
		v += in[bits/8 - i] & (i == 1 ? 0x7f : 0xff); \
	} \
	return sign*(-max-1) + v; \
}

DEFUNPACK_SBE(16, 32767, short)
DEFUNPACK_SBE(32, 2147483647l, long)
#ifdef LLONG_MAX
DEFUNPACK_SBE(64, 9223372036854775807ll, long long)
#endif

DEFUNPACK_SLE(16, 32767, short)
DEFUNPACK_SLE(32, 2147483647l, long)
#ifdef LLONG_MAX
DEFUNPACK_SLE(64, 9223372036854775807ll, long long)
#endif
